/********************************************/
/*                                          */
/*   ATMega_ADF4351_Main.c                  */
/*                                          */
/*   AVR / Arduino Interface to ADF4351     */
/*                                          */
/*   05.02.2021  (c) ADF Tester             */
/*                                          */
/********************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <ctype.h>

#include <avr/interrupt.h>

#include "uart_irq_h.h"
#include "spi_soft_h.h"

/*************************************/
/*           Defines                 */
/*************************************/

/*  Port Defines  */
#ifdef __AVR_ATmega328P__  /*  in I/O.h  */  /*  Arduino Uno / Nano / Micro */
  #define  LED_BIT   0x20
  #define  LED_PORT  PORTB
  #define  LED_DDR   DDRB
#endif
#ifdef __AVR_ATmega2560__  /*  in I/O.h  */  /*  Arduino Mega2560  */
  #define  LED_BIT   0x80
  #define  LED_PORT  PORTB
  #define  LED_DDR   DDRB
#endif
#ifdef __AVR_ATmegaXXXX__  /*  in I/O.h  */  /*  your controller here ...  */
  #define  LED_BIT   0x01
  #define  LED_PORT  PORTA
  #define  LED_DDR   DDRA
#endif

/*  State Machine Defines  */
#define  WAIT_START    11
#define  WAIT_GETSTAT  22
#define  WAIT_DATA     33
#define  DATA_WRITE    44
#define  WAIT_END      55

/*  Protocol Defines  */
#define  START_CHAR    0xFE  /*  Frame Start  */
#define  END_CHAR      0xFD  /*  Frame End    */
#define  READMUX_CHAR  0xF0  /*  Read Mux  */
#define  IDENT_CHAR    0xF1  /*  Identify  */
#define  DATA_CARRIER  0xA0  /*  upper Nibble: 0xA, lower Nibble: Data  */

/*  Data Size (ADFxxxx) */
#define  PAYLOAD_SIZE   4    /* 8 Nibbles --> 4 Bytes per Register for ADF4351  */


/*************************************/
/*            Variables              */
/*************************************/

volatile  uint16_t  ms_ticks = 0;
volatile  uint16_t  timer  = 0;
          uint8_t   raw_idx;
          uint8_t   ii, kk;

uint8_t   rawbuff[16];
uint8_t   data_buf[16];

/*  Status bits Locked & Mux   */
uint8_t   status_msg[8] = {0xFE, 0x00, 0xFD, '\r', '\n', 0, 0, 0};

/*  Identify String   */
const uint8_t  ident_str[16] = {0xFE, 'A','D','F','4','3','5','1', 0xFD, '\r', '\n', 0, 0};

/*  sample register set  */
#define  ADF_NUMREG  6
const uint32_t adf_default[ADF_NUMREG] =
{
   0x00400005,  /*  R5  */
   0x00D195FC,  /*  R4  */
   0x00800323,  /*  R3  */
   0x60028E42,  /*  R2  */
   0x080007D1,  /*  R1  */
   0x02760020   /*  R0  */
};


void Set_ADF4351_default (void);
void ms_tick_wait (uint16_t ticks);


/*************************************/
/*  Example to write a register set  */
/*************************************/

/* FUNCTION **********************************/
void Set_ADF4351_default (void)
{
  int16_t  ii, kk, pos;
  uint8_t  hexval[4];
  union
  { uint32_t word;
    uint8_t  byte[4];
  } u32;

  for (ii=0; ii<ADF_NUMREG; ii++)
  {
    u32.word = adf_default[ii];
    pos = 0;
    for (kk=3; kk>=0; kk--)
    {
      hexval[pos] = u32.byte[kk];
      pos++;
    }
    SPI_Write_ADF4351 (&hexval[0]);
    ms_tick_wait (1);
  }

} /* --- Set_ADF4351_default --- */


/* FUNCTION **********************************/
void toggle_led (void)
{
  static  uint8_t led_on = 0;

  if (led_on == 0)
  {
    LED_PORT = LED_PORT |  LED_BIT;
    led_on = 1;
  }
  else
  {
    LED_PORT = LED_PORT & ~LED_BIT;
    led_on = 0;
  }

} /* --- toggle_led --- */


/* FUNCTION **********************************/
void Timer0_Init (void)
{
  uint8_t  wgm0, wgm1, wgm2;

  /*  WGMx on/off flags set for CTC Mode  */
  wgm0 = 0;     //  TCCR0A  bit 0 -> 0x01  in TCCR0A
  wgm1 = 1;     //  TCCR0A  bit 1 -> 0x02  in TCCR0A
  wgm2 = 0;     //          bit 3 -> 0x08  in TCCR0B

  /*  Set Pin Operation  */
  TCCR0A  = 0x00;  /*  bit 76 = %00  Normal port operation, OC0A pin disconnected  */
  //TCCR0A  = 0x40;  /*  bit 76 = %01  Toggle OC0A pin on Compare Match  */
  //TCCR0A  = 0x80;  /*  bit 76 = %10  Clear OC0A pin on Compare Match   */
  //TCCR0A  = 0xC0;  /*  bit 76 = %11  Set OC0A pin on Compare Match     */

  /*  Timer/Counter Control Register A: set WGM1, WGM0  */
  TCCR0A |= (wgm1 << 1) | (wgm0 << 0);   // wgm0/1 are enable flags

  TCCR0B  = 0x00;   // Initialize
  //TCCR0B  = 0x01;   // bits 001:  prescaler   1  --> 16 MHz
  //TCCR0B  = 0x02;   // bits 010:  prescaler   8  -->  2 MHz
  TCCR0B  = 0x03;   // bits 011:  prescaler  64  --> 250 KHz
  //TCCR0B  = 0x04;   // bits 100:  prescaler 256  --> 62.5 KHz
  //TCCR0B  = 0x05;   // bits 101:  prescaler 1024 --> 15.625 KHz

  /*  Enable Compare Operation  */
  TCCR0B |= 0x80;     // force output compare A register

  /*  set WGM2  */
  TCCR0B |= (wgm2 << 3);  // wgm2 is an enable flag

  OCR0A  = 249;       // set compare A register  250 KHz / 250 --> 1 ms
  TCNT0  =   0;       // set counter

  //TIMSK0 = 0x02;      // enable output compare A IRQ
  TIMSK0 = 1 << OCIE0A;

  ms_ticks = 0;
  timer    = 0;

} /* --- Timer0_Init --- */


/* FUNCTION **********************************/
ISR (TIMER0_COMPA_vect)
{
  ms_ticks++;
  if ((ms_ticks & 0xFF) == 0)
  {
    toggle_led();
  }
  if (timer > 0) timer--;

} /* --- ISR (TIMER0_COMPA_vect) --- */


/* FUNCTION **********************************/
void ms_tick_wait (uint16_t ticks)
{
  timer = ticks;
  do
  {
    // wait
  } while (timer > 0);

} /* --- ms_tick_wait --- */


/* FUNCTION **********************************/
void configure_led_port (void)
{
  LED_DDR  =       0;
  LED_DDR |= LED_BIT;

} /* --- configure_led_port --- */


/* FUNCTION **********************************/
int main (void)
{
#define  TIMEOUT_DEFAULT  10   /*  Timeout in msec  */

  uint8_t  ch, data_avail;
  uint8_t  state;        /*  State Machine  */

  configure_led_port ();
  SPI_Init ();

  Timer0_Init ();   /*  ms_ticks  */
  uart_init ();

  sei();   /*  IRQ required for UART RX  */

  data_avail = 0;
  timer = TIMEOUT_DEFAULT;   /*  Datagram Timeout in msec  */
  state = WAIT_START;

  /*  1 datagram (10 bytes) takes ca 1.625 msec at 57600 Baud  */

  while (1)   /*  State Machine Loop  */
  {
    ch = 0;
    switch  (state)
    {
      /*  scan for data frame start (START_CHAR)  */
      case WAIT_START:
          timer = TIMEOUT_DEFAULT;  /*  timer decrement in ISR  */
          data_avail = uart_char_available ();
          if (data_avail != 0)
          {
            ch = uart_getchar ();

            if      (ch == START_CHAR)  /*  Frame start  */
            {
              raw_idx = 0;
              state = WAIT_DATA;
            }
            else if (ch == READMUX_CHAR)  /*  Send Status Bits  */
            {
              status_msg[1] = 0xA0 | ADF_GetBits ();
              uart_putstr ((char*)&status_msg[0]);
            }
            else if (ch == IDENT_CHAR)   /*  Send Identification  */
            {
              uart_putstr ((char*)&ident_str[0]);
            }
            else if (ch == '!')   /*  Send Test Output  */
            {
              uart_putstr ((char*)"Hallo ... luft ...\r\n");
            }
          }
      break;


      /*  data frame fillup  */
      case WAIT_DATA:
          data_avail = uart_char_available ();
          if (data_avail != 0)
          {
            ch = uart_getchar ();
            if (ch != 0)
            {
              rawbuff[raw_idx] = ch & 0x0F; /*  mask off upper nibble  */
              raw_idx++;
              data_avail = 0;
              if (raw_idx >= (PAYLOAD_SIZE*2))
              {
                state = DATA_WRITE;
              }
            }
          }
          if (timer == 0) /*  check timeout  */
          {
            state = WAIT_START;
          }
      break;


      /*  data frame complete  */
      case DATA_WRITE:
          kk = 0;
          for (ii=0; ii<PAYLOAD_SIZE; ii++)
          {
            data_buf[ii] = (rawbuff[kk] << 4 ) | (rawbuff[kk+1]);
            kk = kk + 2; /*  Rawbuffer Index  */
          }
          /*  1 Datagram ADF4351 dauert ca 35 usec mit Soft SPI  */
          SPI_Write_ADF4351 (&data_buf[0]); /*  4 bytes + Latch Enable  */
          state = WAIT_END;
      break;


      /*  "frame end" delimiter  */
      case WAIT_END:
          data_avail = uart_char_available ();
          if (data_avail != 0)
          {
            ch = uart_getchar ();
            if (ch == END_CHAR)  /*  Frame end  */
            {
              state = WAIT_START;
            }
          }
          if (timer == 0) /*  check timeout  */
          {
            state = WAIT_START;
          }
      break;

      /*  optional: illegal state  */
      default:
          state = WAIT_START;
      break;

    } /* --- switch --- */


  } /* --- while (1) --- */


} /* --- main --- */



/**************************************************/
/* ----- end of file ATMega_ADF4351_Main.c ------ */
/**************************************************/
