/******************************************************
*******************************************************
**                                                   **
**  MODULE    spi_soft.c                             **
**                                                   **
*******************************************************
******************************************************/

#include <avr/io.h>
#include "spi_soft_h.h"

/*********************************************/
/*  Port Settings Arduino Mega328P Soft SPI  */
/*  AVR 5V to ADFxxxx 3.3 V requires level shifter!  */

#ifdef __AVR_ATmega328P__  /*  in I/O.h  */

#define PORT_SPI        PORTC  /*  Soft SPI  */
#define DDR_SPI         DDRC   /*  Soft SPI  */
#define PIN_SPI         PINC   /*  Soft SPI  ADF4351 MuxOut & Locked Bit */

#define SCK_BIT         (1<<PORTC0)  /* 0x01  Port C0  Mega328P Pin 23  Arduino Pin A0 */
#define MOSI_BIT        (1<<PORTC1)  /* 0x02  Port C1  Mega328P Pin 24  Arduino Pin A1 */
#define ADF_LE_BIT      (1<<PORTC2)  /* 0x04  Port C2  Mega328P Pin 25  Arduino Pin A2 */
#define ADF_MUX_BIT     (1<<PINC3)   /* 0x08  Port C3  Mega328P Pin 26  Arduino Pin A3 */
#define ADF_LOCKED_BIT  (1<<PINC4)   /* 0x10  Port C4  Mega328P Pin 27  Arduino Pin A4 */
#warning  Mega328P
#endif  /****__AVR_ATmega328P__***************/


/*********************************************/
/*  Port Settings Arduino Mega2560 Soft SPI  */
/*  AVR 5V to ADFxxxx 3.3 V requires level shifter!  */

#ifdef __AVR_ATmega2560__  /*  in I/O.h  */

#define PORT_SPI        PORTF  /*  Soft SPI  */
#define DDR_SPI         DDRF   /*  Soft SPI  */
#define PIN_SPI         PINF   /*  Soft SPI  ADF4351 MuxOut & Locked Bit */

#define SCK_BIT         (1<<PORTF0)  /* 0x01  Port F0  Mega2560 Pin 54  Arduino Pin A0 */
#define MOSI_BIT        (1<<PORTF1)  /* 0x02  Port F1  Mega2560 Pin 55  Arduino Pin A1 */
#define ADF_LE_BIT      (1<<PORTF2)  /* 0x04  Port F2  Mega2560 Pin 56  Arduino Pin A2 */
#define ADF_MUX_BIT     (1<<PINF3)   /* 0x08  Port F3  Mega2560 Pin 57  Arduino Pin A3 */
#define ADF_LOCKED_BIT  (1<<PINF4)   /* 0x10  Port F4  Mega2560 Pin 58  Arduino Pin A4 */
#warning  Mega2560
#warning  -------- Achtung --------
#warning  JTAG disablen wenn PORTF!
#warning  (PF4...PF7 --> JTAG)
#warning  -------- Achtung --------
#endif  /****__AVR_ATmega2560__***************/


/*********************************************/
/*  Soft SPI macros  */

#define ADF_LE_HIGH    PORT_SPI  |=  (ADF_LE_BIT)
#define ADF_LE_LOW     PORT_SPI  &= ~(ADF_LE_BIT)

#define SPI_SCK_HIGH   PORT_SPI  |=  (SCK_BIT);
#define SPI_SCK_LOW    PORT_SPI  &= ~(SCK_BIT);

#define ADF_GETLOCKED  (PIN_SPI  & ADF_LOCKED_BIT)
#define ADF_GETMUX     (PIN_SPI  & ADF_MUX_BIT)

#define  NOP0  ;
#define  NOP1  asm volatile ( "nop" "\n\t"  );


/* FUNCTION **********************************/
/*    Soft SPI Init     */
void SPI_Init (void)
{
  DDR_SPI  &= ~(ADF_MUX_BIT   );  //  DDR: MUX Input
  DDR_SPI  &= ~(ADF_LOCKED_BIT);  //  DDR: LOCKED Input
  DDR_SPI  |=  (SCK_BIT   );      //  DDR: SCLOCK output
  DDR_SPI  |=  (MOSI_BIT  );      //  DDR: MOSI output
  DDR_SPI  |=  (ADF_LE_BIT);      //  DDR: LE output

  ADF_LE_LOW;    //  latch enable low, default inactive
  SPI_SCK_LOW;   //  SCLOCK default inactive

} /* --- SPI_Init --- */


/* FUNCTION **********************************/
uint8_t ADF_GetBits (void)
{
  uint8_t  input = 0;

  if ( (ADF_GETMUX)    != 0)
  { input |= 0x02;
  }
  if ( (ADF_GETLOCKED) != 0)
  { input |= 0x01;
  }

  return input;

} /* --- ADF_GetBits --- */


/* FUNCTION **********************************/
void SPI_WriteByte (uint8_t val)
{
  uint8_t  ii;

  for (ii=0; ii<8; ii++)
  {
    if (val & 0x80) PORT_SPI |=  (MOSI_BIT);
    else            PORT_SPI &= ~(MOSI_BIT);
    val <<= 1;

    SPI_SCK_HIGH;  // clock high
    NOP1;
    SPI_SCK_LOW;   // clock low
  }

} /* --- SPI_WriteByte --- */


/* FUNCTION **********************************/
void SPI_Write_ADF4351 (uint8_t *p_frame)
{
  SPI_WriteByte (*p_frame);  p_frame++;
  SPI_WriteByte (*p_frame);  p_frame++;
  SPI_WriteByte (*p_frame);  p_frame++;
  SPI_WriteByte (*p_frame);

  ADF_LE_HIGH;  // latch enable high
  NOP1;
  ADF_LE_LOW ;  // latch enable low

} /* --- SPI_Write_ADF4351 --- */


/* -------------------------------------------- */
/* ---------- end of file spi_soft.c ---------- */
/* -------------------------------------------- */
