/******************************************************
*******************************************************
**                                                   **
**  MODULE    uart_irq.c                             **
**                                                   **
*******************************************************
******************************************************/

#include <ctype.h>
#include <avr/io.h>
#include <avr/interrupt.h>

/* Useful macros */

#define TX_START()    UCSR0B |=  _BV(TXEN0)   // Enable TX
#define TX_STOP()     UCSR0B &= ~_BV(TXEN0)   // Disable TX
#define RX_START()    UCSR0B |=  _BV(RXEN0)   // Enable RX
#define RX_STOP()     UCSR0B &= ~_BV(RXEN0)   // Disable RX

/* Interrupt macros; Remember to set the GIE bit in SREG before using (see datasheet) */

#define RX_INT_ENABLE()   UCSR0B |=  _BV(RXCIE0)  // Enable interrupt on RX complete
#define RX_INT_DISABLE()  UCSR0B &= ~_BV(RXCIE0)  // Disable RX interrupt
#define TX_INT_ENABLE()   UCSR0B |=  _BV(TXCIE0)  // Enable interrupt on TX complete
#define TX_INT_DISABLE()  UCSR0B &= ~_BV(TXCIE0)  // Disable TX interrupt

#define  RX_BUFF_SIZE   80
static unsigned char rx_buffer[RX_BUFF_SIZE];
static volatile unsigned char* p_read;  // buffer readout pointer
static volatile unsigned char* p_write; // receive buffer pointer


/* FUNCTION **********************************/
void uart_init (void)
{
//#define BAUD  9600UL   // Baudrate
//#define BAUD 19200UL   // Baudrate
//#define BAUD 38400UL   // Baudrate
#define BAUD 57600UL   // Baudrate
//#define BAUD 115200UL  // Baudrate

// Baudrate Berechnungen
#define UBRR_VAL   ( ((F_CPU+BAUD*8) / (BAUD*16)) - 1)  // clever runden
#define BAUD_REAL  (  F_CPU / (16 * (UBRR_VAL+1)))      // Reale Baudrate
#define BAUD_ERROR ( (BAUD_REAL*1000)/BAUD)             // Fehler in Promille, 1000 = kein Fehler.
#if ( (BAUD_ERROR<990) || (BAUD_ERROR>1010))
//#error Systematischer Fehler der Baudrate groesser 1% und damit zu hoch!
#endif

  /* Set baud rate */

  UBRR0H = UBRR_VAL >> 8;
  UBRR0L = UBRR_VAL & 0xFF;

  UCSR0C = (1<<UCSZ01) | (1<<UCSZ00);  /*  asynch 8N1  */

  TX_START();  /*  start transmitter  */
  RX_START();  /*  start receiver  */

  p_write = rx_buffer;  /*  set ring buffer pointer  */
  p_read  = rx_buffer;

  RX_INT_ENABLE();  /*  enable receive interrupt  */

} /* --- uart_init --- */


/* FUNCTION **********************************/

#ifdef __AVR_ATmega328P__
ISR (USART_RX_vect)
#endif
#ifdef __AVR_ATmega2560__
ISR (USART0_RX_vect)
#endif
{
  *p_write = UDR0;   /*  read char  */

  p_write++;
  if (p_write == (rx_buffer+RX_BUFF_SIZE))
  {
    p_write = rx_buffer;   /*  wrap around  */
  }

} /* --- ISR (USART_RX_vect) --- */


/* FUNCTION **********************************/
uint8_t uart_char_available (void)
{
  if (p_write >= p_read)  /*  normal  */
  {
    return p_write - p_read;
  }
  else                    /*  wrap around  */
  {
    return RX_BUFF_SIZE - (p_read - p_write);
  }

} /* --- uart_char_available --- */


/* FUNCTION **********************************/
uint8_t uart_getchar (void)
{
  static uint8_t ch;

  ch = 0;
  if (p_write != p_read)
  {
    ch = *p_read;
    p_read++;
    if (p_read == (rx_buffer+RX_BUFF_SIZE))
    {
      p_read = rx_buffer; /*  wrap around  */
    }
  }
  return ch;

} /* --- uart_getchar --- */


/* FUNCTION **********************************/
void uart_putchar (char ch)
{
  while (! (UCSR0A & (1<<UDRE0)))
  {
  }
  UDR0 = ch;

} /* --- uart_putchar --- */


/* FUNCTION **********************************/
void uart_putstr (char *str)
{
  while (*str != '\0')
  {
    uart_putchar (*str);
    ++str;
  }

} /* --- uart_putstr --- */


/******************************************/
/* ------- end of file uart_irq.c ------- */
/******************************************/

